SystemOrganization addCategory: #'Zinc-Resource-Meta-Core'!

Object subclass: #ZnMimeType
	instanceVariableNames: 'main sub parameters'
	classVariableNames: 'ExtensionsMap MimeTypes'
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnMimeType commentStamp: '<historical>' prior: 0!
I am ZnMimeType.
Mime types are an element used in describing resources, more specifically their format.

Here are some examples MIME types:

	text/plain
	text/html
	text/plain;charset=utf-8
	image/png
	text/*
 
A mime type has a main/sub form with optional parameters. 

For more information: http://en.wikipedia.org/wiki/MIME_type
I know whether I am binary or not and know my charset when applicable.
My class side offers access to some common constants.
Note that for text types we default to UTF-8 encoding.

Examples:

	ZnMimeType textPlain = 'text/plain' asZnMimeType.
	ZnMimeType textPlain charSet.
	ZnMimeType textHtml matches: ZnMimeType text.
	ZnMimeType default matches: ZnMimeType any.
	ZnMimeType imagePng isBinary.
	ZnMimeType forFilenameExtension: 'html'.

I started life as a copy of WAMineType.

Part of Zinc HTTP Components.!

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 19:59'!
any
	^ self predefinedInstanceAt: #any
	! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 19:59'!
applicationFormUrlEncoded
	^ self predefinedInstanceAt: #applicationFormUrlEncoded! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 19:59'!
applicationJavascript
	^ self predefinedInstanceAt: #applicationJavascript! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 19:59'!
applicationJson
	^ self predefinedInstanceAt: #applicationJson! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 19:59'!
applicationOctetStream
	^ self predefinedInstanceAt: #applicationOctetStream! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/27/2012 18:42'!
applicationPdf
	^ self predefinedInstanceAt: #applicationPdf! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
applicationXml
	^ self predefinedInstanceAt: #applicationXml! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 15:48'!
default
	^ self applicationOctetStream ! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/16/2010 20:29'!
forFilenameExtension: extension
	^ ExtensionsMap 
		at: extension 
		ifAbsent: [ self default ]! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/8/2012 14:17'!
fromString: aString 
	| main endOfSub sub parts parameters |
	main := aString copyUpTo: $/.
	endOfSub := aString indexOf: $;.
	endOfSub := endOfSub = 0 
		ifTrue: [ aString size ]
		ifFalse: [ endOfSub - 1 ].
	sub := aString copyFrom: main size + 2 to: endOfSub.
	endOfSub = aString size ifTrue: [ ^ self main: main sub: sub ].
	parts := (aString copyFrom: endOfSub + 1 to: aString size) subStrings: ';'.
	parameters := Dictionary new.
	parts do: [ :each | 
		parameters
			at: (each copyUpTo: $=) trimBoth 
			put: (each copyAfter: $=) trimBoth ].
	^ self 
		main: main
		sub: sub
		parameters: parameters! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
imageGif
	^ self predefinedInstanceAt: #imageGif! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
imageJpeg
	^ self predefinedInstanceAt: #imageJpeg! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
imagePng
	^ self predefinedInstanceAt: #imagePng! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/27/2012 14:35'!
imageSvg
	^ self predefinedInstanceAt: #imageSvg
! !

!ZnMimeType class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 9/16/2010 20:27'!
initialize
	self initializeMimeTypes.
	self initializeExtensionsMap! !

!ZnMimeType class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2012 14:36'!
initializeExtensionsMap 
	"To be extended later"
	
	ExtensionsMap := Dictionary new
		at: 'txt' put: self textPlain;
		at: 'text' put: self textPlain;
		at: 'htm' put: self textHtml;
		at: 'html' put: self textHtml;
		at: 'gif' put: self imageGif;
		at: 'jpg' put: self imageJpeg;
		at: 'jpeg' put: self imageJpeg;
		at: 'png' put: self imagePng;
		at: 'js' put: self applicationJavascript;
		at: 'xml' put: self applicationXml;
		at: 'css' put: self textCss;
		at: 'pdf' put: self applicationPdf;
		at: 'svg' put: self imageSvg;
		yourself! !

!ZnMimeType class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2012 14:36'!
initializeMimeTypes 
	"To be extended later"

	MimeTypes := Dictionary new
		at: #applicationOctetStream put: (self main: 'application' sub: 'octet-stream');
		at: #applicationJavascript put: (self main: 'application' sub: 'javascript');
		at: #applicationJson put: (self main: 'application' sub: 'json');
		at: #applicationXml put: (self main: 'application' sub: 'xml');
		at: #applicationPdf put: (self main: 'application' sub: 'pdf');
		at: #applicationFormUrlEncoded put: (self main: 'application' sub: 'x-www-form-urlencoded');
		at: #imageGif put: (self main: 'image' sub: 'gif');
		at: #imageJpeg put: (self main: 'image' sub: 'jpeg');
		at: #imagePng put: (self main: 'image' sub: 'png');
		at: #imageSvg put: (self main: 'image' sub: 'svg+xml');
		at: #textPlain put: ((self main: 'text' sub: 'plain') setCharSetUTF8; yourself);
		at: #textHtml put: ((self main: 'text' sub: 'html') setCharSetUTF8; yourself);
		at: #textCss put: (self main: 'text' sub: 'css');
		at: #textJavascript put: (self main: 'text' sub: 'javascript');
		at: #multiPartFormData put: (self main: 'multipart' sub: 'form-data');
		at: #any put: (self main: '*' sub: '*');
		at: #text put: (self main: 'text' sub: '*');
		yourself! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:22'!
main: aMainString sub: aSubString
	^ self new
		main: aMainString;
		sub: aSubString;
		yourself! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:22'!
main: aMainString sub: aSubString parameters: aDictionary
	^ (self main: aMainString sub: aSubString)
		parameters: aDictionary;
		yourself! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
multiPartFormData
	^ self predefinedInstanceAt: #multiPartFormData! !

!ZnMimeType class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/6/2011 20:43'!
predefinedInstanceAt: key
	"Look up and return a predefined instance known under key.
	I return a copy that can be modified.
	See #initializeMimeTypes for the full list."
	
	^ (self sharedInstanceAt: key) copy! !

!ZnMimeType class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/6/2011 19:58'!
sharedInstanceAt: key
	"Look up and return a predefined instance known under key.
	See #initializeMimeTypes for the full list.
	The returned instance should not be modified."
	
	^ MimeTypes at: key! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
text
	^ self predefinedInstanceAt: #text
	! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
textCss
	^ self predefinedInstanceAt: #textCss! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
textHtml
	^ self predefinedInstanceAt: #textHtml! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:01'!
textJavascript
	^ self predefinedInstanceAt: #textJavascript! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:01'!
textPlain
	^ self predefinedInstanceAt: #textPlain! !

!ZnMimeType methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:22'!
= other
	^ (self class == other class)
		and: [ self main = other main
		and: [ self sub = other sub ] ]! !

!ZnMimeType methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/4/2011 19:47'!
asZnMimeType
	^ self! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
charSet
	^ self parameterAt: 'charset' ifAbsent: [ nil ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 12/2/2010 11:24'!
charSet: aString
	self parameters at: 'charset' put: aString! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 3/6/2012 10:54'!
clearCharSet
	self removeParameter: 'charset'! !

!ZnMimeType methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:22'!
hash
	^ self main hash bitXor: self sub hash! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:22'!
isBinary
	"answers whether the contents of a document of the receiving mime type are binary"
	| subTypes |
	self main = 'text' ifTrue: [ ^ false ].
	self = self class applicationJson ifTrue: [ ^ false ].
	self main = 'application' ifFalse: [ ^ true ].
	subTypes := self sub subStrings: '+'.
	^ subTypes noneSatisfy: [ :each | 
		#('x-javascript' 'xml') includes: each ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/2/2010 11:19'!
isCharSetUTF8
	^ self charSet notNil and: [ self charSet sameAs: 'utf-8' ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:22'!
isNonStandard
	"tests if the receiver is a non-standard mime type that is not registered with IANA"
	^ (self main beginsWith: 'x-')
		or: [ (self main beginsWith: 'X-')
		or: [ (self sub beginsWith: 'x-')
		or: [ self sub beginsWith: 'X-' ] ] ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:45'!
isVendorSpecific
	"tests if the receiver is a vendor specific mime type"
	
	^ self sub beginsWith: 'vnd.'! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
main
	^ main! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
main: aString
	main := aString! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/8/2012 13:14'!
matches: aMimeType
	"aMimeType is the pattern to match, it is a normal instance where main or sub can be wildcards"
	
	^ (aMimeType main = '*'
		or: [ aMimeType main = self main ])
		and: [
			aMimeType sub = '*'
				or: [ aMimeType sub = self sub ] ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 3/6/2012 11:00'!
parameterAt: key
	^ self parameterAt: key ifAbsent: [ KeyNotFound signalFor: key ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 3/6/2012 10:53'!
parameterAt: key ifAbsent: block
	parameters isNil ifTrue: [ ^ block value ].
	^ self parameters at: key ifAbsent: block! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 9/7/2010 14:50'!
parameterAt: aKey put: aValue
	self parameters at: aKey put: aValue! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/6/2011 20:40'!
parameters
	parameters isNil ifTrue: [ parameters := SmallDictionary new ].
	^ parameters! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
parameters: aDictionary
	parameters := aDictionary! !

!ZnMimeType methodsFor: 'copying' stamp: 'SvenVanCaekenberghe 12/6/2011 20:41'!
postCopy
	super postCopy.
	parameters := parameters copy! !

!ZnMimeType methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/11/2010 20:49'!
printOn: stream
	stream
		nextPutAll: self main;
		nextPut: $/;
		nextPutAll: self sub.
	parameters isNil ifFalse: [ 
		parameters keysAndValuesDo: [ :key :value | 
			stream
				nextPut: $;;
				nextPutAll: key;
				nextPut: $=;
				nextPutAll: value ] ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 3/6/2012 10:53'!
removeParameter: key
	parameters isNil ifTrue: [ ^ self ].
	^ self parameters removeKey: key ifAbsent: [ nil ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 12/2/2010 11:20'!
setCharSetUTF8
	self charSet: 'utf-8'! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
sub
	^ sub! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
sub: aString
	sub := aString! !

Object subclass: #ZnResourceMetaUtils
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnResourceMetaUtils commentStamp: '<historical>' prior: 0!
I am ZnResourceMetaUtils.

I contain some class side utility methods.

Part of Zinc HTTP Components.!

!ZnResourceMetaUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 12/17/2012 13:08'!
decodePercent: string
	^ ZnPercentEncoder new decode: string! !

!ZnResourceMetaUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 12/17/2012 13:07'!
encodePercent: string withEncoding: encoding
	^ ZnPercentEncoder new
		characterEncoder: (ZnCharacterEncoder newForEncoding: encoding);
		encode: string! !

!ZnResourceMetaUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 12/8/2012 20:46'!
parseQueryFrom: stream
	| string index fields |
	fields := ZnMultiValueDictionary new.
	[ stream atEnd ] whileFalse: [
		string := stream upTo: $&.
		index := string indexOf: $=.
		index > 0
			ifFalse: [ fields at: (self decodePercent: string) put: nil ]
			ifTrue: [
				fields
					at: (self decodePercent: (string copyFrom: 1 to: index - 1))
					add: (self decodePercent: (string copyFrom: index + 1 to: string size)) ] ].
	^ fields! !

!ZnResourceMetaUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 12/8/2012 20:47'!
writeQueryFields: dictionary on: stream
	self writeQueryFields: dictionary withTextEncoding: 'utf-8' on: stream ! !

!ZnResourceMetaUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 12/17/2012 13:07'!
writeQueryFields: dictionary withTextEncoding: encoding on: stream
	| first |
	first := true.
	dictionary keysAndValuesDo: [ :key :value |
		first ifFalse: [ stream nextPut: $& ].
		first := false.
		stream nextPutAll: (self encodePercent: key withEncoding: encoding).
		value isNil ifFalse: [
			stream 
				nextPut: $=;
				nextPutAll: (self encodePercent: value withEncoding: encoding) ] ]! !

Object subclass: #ZnUrl
	instanceVariableNames: 'scheme host port segments query fragment username password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnUrl commentStamp: '<historical>' prior: 0!
I am ZnUrl, an implementation of an interpreted URL/URI.
URLs are an element used in describing resources, more specifically to identify them.

I consist of the following parts:
  - scheme - like #http, #https, #ws, #wws, #file or nil
  - host - hostname string or nil
  - port - port integer or nil
  - segments - collection of path segments, ends with #/ for directories
  - query - query dictionary or nil
  - fragment - fragment string or nil
  - username - username string or nil
  - password - password string or nil

The syntax of my external representation informally looks like this

  scheme://username:password@host:port/segments?query#fragment

I am most often created by parsing my external representation using either my #fromString: class method or by sending the #asZnUrl convenience method to a String. Using #asZnUrl helps in accepting both Strings and ZnUrls arguments.

  ZnUrl fromString: 'http://www.google.com/search?q=Smalltalk'.

I can also be constucted programmatically.

  ZnUrl new 
    scheme: #https; 
    host: 'encrypted.google.com'; 
    addPathSegment: 'search'; 
    queryAt: 'q' put: 'Smalltalk'; 
    yourself.
  
My components can be manipulated destructively. Here is an example:

  ('http://www.google.com/?one=1&two=2' asZnUrl)
    queryAt: 'three' put: '3';
    queryRemoveKey: 'one';
    yourself.

Some characters of parts of a URL are illegal because they would interfere with the syntax and further processing and thus have to be encoded. The methods in accessing protocols do not do any encoding, those in parsing and printing do. Here is an example:

  ('http://www.google.com' asZnUrl)
    addPathSegment: 'some encoding here';
    queryAt: 'and some encoding' put: 'here, too';
    yourself

My parser is somewhat forgiving and accepts some unencoded URLs as well, like most browsers would.

  'http://www.example.com:8888/a path?q=a, b, c' asZnUrl.

I can parse in the context of a default scheme, like a browser would do.

  ZnUrl fromString: 'www.example.com' defaultScheme: #http

Given a scheme, I know its default port, try #portOrDefault.

A path defaults to what is commonly referred to as slash, test with #isSlash. Paths are most often (but don't have to be) interpreted as filesystem paths. To support this, I have #isFile and #isDirectory tests and #file and #directory accessors.

I have some support to handle one URL in the context of another one, this is also known as a relative URL in the context of an absolute URL. Refer to #isAbsolute, #isRelative and #inContextOf:

  '/folder/file.txt' asZnUrl inContextOf: ('http://fileserver.example.net:4400' asZnUrl).

Sometimes, the combination of my host and port are referred to as authority, see #authority.

URL/URI/URN (Uniform/Universal Resource Locator/Identifier/Name) are closely related and can be and are used as synonyms is many contexts. Refer to http://en.wikipedia.org/wiki/Url for more information.

There is a convenience method #retrieveContents to download the resource a ZnUrl points to,

  'http://zn.stfx.eu/zn/numbers.txt' asZnUrl retrieveContents.
  
Part of Zinc HTTP Components.!

!ZnUrl class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/8/2012 15:26'!
defaultPortForScheme: scheme
	scheme = #file ifTrue: [ ^ nil ].
	(#(http ws) includes: scheme) ifTrue: [ ^ 80 ].
	(#(https wss) includes: scheme)  ifTrue: [ ^ 443 ].
	(ZnUnknownScheme scheme: scheme) signal! !

!ZnUrl class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/3/2011 19:35'!
fromString: string
	^ (self new)
		parseFrom: string; 
		yourself ! !

!ZnUrl class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/11/2011 14:47'!
fromString: string defaultScheme: defaultScheme
	^ (self new)
		parseFrom: string defaultScheme: defaultScheme; 
		yourself ! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/9/2012 16:27'!
& association
	^ self withQuery: association ! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 3/16/2013 20:22'!
/ object
	^ object addedToZnUrl: self! !

!ZnUrl methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 1/4/2011 11:00'!
= anObject
	^ self class == anObject class 
		and: [ self equals: anObject ]! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/9/2012 16:27'!
? association
	^ self withQuery: association ! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 7/9/2012 16:18'!
addPathSegment: segment
	"Modify the receiver's path by adding segment at the end"
	
	segments isNil ifTrue: [ 
		segments := OrderedCollection new ].
	segments add: segment! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 7/9/2012 16:18'!
addPathSegments: pathSegments
	"Modify the receiver's path by adding the elements of pathSegments at the end"

	pathSegments do: [ :each | self addPathSegment: each ]! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 3/16/2013 20:39'!
addedToZnUrl: url
	^ url withPathSegments: self pathSegments ! !

!ZnUrl methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/6/2013 19:21'!
asRelativeUrl
	"Copy the receiver, with scheme, host and port cleared"
	
	^ self copy
		scheme: nil;
		host: nil;
		port: nil;
		yourself! !

!ZnUrl methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/4/2011 11:24'!
asZnUrl
	^ self! !

!ZnUrl methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 8/11/2011 12:05'!
asZnUrlWithDefaults
	^ self copy 
		setDefaults; 
		yourself! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 21:01'!
authority
	^ self hasNonDefaultPort 
		ifTrue: [ 
			String streamContents: [ :stream | 
				stream nextPutAll: self host; nextPut: $:; print: self port ] ]
		ifFalse: [ 
			self host ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 3/16/2013 19:32'!
clearPath
	self hasPath
		ifTrue: [ segments removeAll ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/4/2011 11:56'!
closePath
	self isSlash
		ifFalse: [ self addPathSegment: #/ ]! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/30/2013 16:27'!
decodePercent: string
	^ ZnResourceMetaUtils decodePercent: string! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 3/16/2013 19:55'!
directory
	^ self isDirectoryPath
		ifTrue: [ self path ]
		ifFalse: [ 
			String streamContents: [ :stream |
				segments allButLast  
					do: [ :each | stream nextPutAll: each ]
					separatedBy: [ stream nextPut: $/ ] ] ]! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/17/2012 13:10'!
encode: string on: stream
	stream nextPutAll: (ZnPercentEncoder new encode: string)! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/30/2013 17:23'!
enforceKnownScheme
	(scheme isNil or: [ #(#http #https #ws #wss #file) includes: scheme ])
		ifFalse: [ (ZnUnknownScheme scheme: scheme) signal ]! !

!ZnUrl methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:59'!
equals: url
	self scheme = url scheme ifFalse: [ ^ false ].
	self host = url host ifFalse: [ ^ false ].
	self portOrDefault = url portOrDefault ifFalse: [ ^ false ].
	((self isSlash and: [ url isSlash ]) or: [ self segments = url segments ]) ifFalse: [ ^ false ].
	self query = url query ifFalse: [ ^ false ].
	self fragment = url fragment ifFalse: [ ^ false ].
	^ true! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 3/16/2013 19:39'!
file
	^ self isDirectoryPath
		ifTrue: [ String new ]
		ifFalse: [ segments last ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/4/2011 14:46'!
firstPathSegment
	^ self isSlash
		ifTrue: [ nil ]
		ifFalse: [ segments first ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:51'!
fragment
	^ fragment! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:51'!
fragment: string
	fragment := string! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasFragment
	^ fragment notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasHost
	^ host notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 21:01'!
hasNonDefaultPort
	^ self hasPort 
		and: [ self port ~= (self class defaultPortForScheme: self schemeOrDefault) ]! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:19'!
hasPassword
	^ password notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 12:03'!
hasPath
	^ segments notNil and: [ segments isEmpty not ]! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasPort
	^ port notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/6/2013 20:12'!
hasQuery
	^ query notNil and: [ query isEmpty not ]! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasScheme
	^ scheme notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:19'!
hasUsername
	^ username notNil! !

!ZnUrl methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:43'!
hash
	^ self hasPath
		ifTrue: [ segments inject: host hash into: [ :sum :each | sum bitXor: each hash ] ]
		ifFalse: [ host hash ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:09'!
host
	^ host! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/6/2013 19:19'!
host: hostName
	host := hostName ifNil: [ nil ] ifNotNil: [ hostName asLowercase ]! !

!ZnUrl methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/7/2013 09:59'!
inContextOf: absoluteUrl
	"Return a copy of the receiver where scheme, host and port
	are taken from absoluteUrl unless they are already in the receiver.
	Path merging is not supported."
	
	| copy |
	copy := self copy.
	copy hasScheme ifFalse: [ copy scheme: absoluteUrl scheme ].
	copy hasHost ifFalse: [ copy host: absoluteUrl host ].
	(copy hasPort not and: [ copy scheme = absoluteUrl scheme ]) 
		ifTrue: [ copy port: absoluteUrl port ].      
	^ copy! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/7/2013 09:59'!
isAbsolute
	"We consider URLs with scheme://host as absolute (port is not relevant here).
	See also #inContextOf: Path merging is not supported."
	
	^ self hasScheme and: [ self hasHost ]! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/14/2013 11:36'!
isDirectoryPath
	^ self hasPath 
		ifTrue: [ segments last = #/ ]
		ifFalse: [ true ]
	! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/30/2013 17:15'!
isEmpty
	^ (self hasScheme | self hasHost | self hasPath | self hasQuery | self hasFragment | self hasUsername | self hasPassword) not! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/14/2013 11:42'!
isFile
	^ scheme = #file! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/14/2013 11:37'!
isFilePath
	^ self isDirectoryPath not! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:50'!
isHttp
	^ scheme == #http! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:50'!
isHttps
	^ scheme == #https! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/13/2011 10:41'!
isLocalHost
	^ self hasHost and: [ #('localhost' '127.0.0.1') includes: self host ]! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:55'!
isRelative
	^ self isAbsolute not! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:54'!
isSlash
	^ self hasPath not or: [ segments size = 1 and: [ segments first = #/ ] ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/4/2011 14:51'!
lastPathSegment
	^ self isSlash
		ifTrue: [ nil ]
		ifFalse: [ segments last ]! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:41'!
parseAuthority: string from: start to: stop
	| index |
	(index := string indexOf: $@ startingAt: start) > 0
		ifTrue: [
			self parseUserInfo: (ReadStream on: string from: start to: index - 1).
			self parseHostPort: (ReadStream on: string from: index + 1 to: stop ) ]
		ifFalse: [ 
			self parseHostPort: (ReadStream on: string from: start to: stop ) ]! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 8/11/2011 14:47'!
parseFrom: string
	self parseFrom: string defaultScheme: nil! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:39'!
parseFrom: string defaultScheme: defaultScheme
	| start end index |
	start := 1.
	end := string size.
	(index := string indexOf: $#) > 0
		ifTrue: [ 
			self fragment: (self decodePercent: (string copyFrom: index + 1 to: end)).
			end := index - 1 ].
	(index := string indexOf: $?) > 0
		ifTrue: [
			self query: (ZnResourceMetaUtils parseQueryFrom: (ReadStream on: string from: index + 1 to: end)).
			end := index - 1 ].
	(index := string indexOfSubCollection: '://') > 0
		ifTrue: [
			self scheme: (string copyFrom: 1 to: index - 1).
			start := index + 3 ]
		ifFalse: [ 
			defaultScheme ifNotNil: [ self scheme: defaultScheme ] ].
	self hasScheme 
		ifTrue: [  
			(index := string indexOf: $/ startingAt: start) > 0
				ifTrue: [
					self parseAuthority: string from: start to: index - 1.
					start := index ]
				ifFalse: [ 
					^ self parseAuthority: string from: start to: end ] ].
	self parsePath: (ReadStream on: string from: start to: end)! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 2/6/2013 18:20'!
parseHostPort: stream
	| hostString |
	self isFile
		ifTrue: [ 
			(hostString := stream upTo: $/) isEmpty
				ifFalse: [ self host: (self decodePercent: hostString) ] ]
		ifFalse: [ 
			(hostString := stream upTo: $:) isEmpty
				ifFalse: [ self host: (self decodePercent: hostString) ].
			stream atEnd
				ifFalse: [ self port: (Integer readFrom: stream ifFail: [ ZnPortNotANumber signal ]) ] ]! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:28'!
parsePath: stream
	stream peekFor: $/.
	[ stream atEnd ] whileFalse: [ | segment |
		segment := String streamContents: [ :stringStream |
			[ stream atEnd not and: [ stream peek ~= $/ ] ] whileTrue: [
				stringStream nextPut: stream next ] ].
		(segment isEmpty or: [ segment = '.' ])
			ifFalse: [
				segment = '..' 
					ifTrue: [ self removeLastPathSegment ]
					ifFalse: [ self addPathSegment: (self decodePercent: segment) ] ].
		((stream peekFor: $/) and: [ stream atEnd ])
			ifTrue: [ self closePath ] ]! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:28'!
parseUserInfo: stream
	| userString |
	(userString := stream upTo: $:) isEmpty 
		ifFalse: [  
			self username: (self decodePercent: userString) ].
	stream atEnd 
		ifFalse: [ 
			self password: (self decodePercent: stream upToEnd) ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:42'!
password
	^ password! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:19'!
password: string
	password := string! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 3/16/2013 19:43'!
path
	self hasPath ifFalse: [ ^ String new ].
	^ String streamContents: [ :stream | 
		segments 
			do: [ :each | 
				each == #/
					ifFalse: [ stream nextPutAll: each ] ]
			separatedBy: [ stream nextPut: $/ ] ]! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 11:16'!
pathPrintString
	^ String streamContents: [ :stream | 
			self printPathOn: stream ]! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 11:17'!
pathQueryFragmentPrintString
	^ String streamContents: [ :stream | 
			self printPathQueryFragmentOn: stream ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/4/2011 14:51'!
pathSegments
	^ segments! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:09'!
port
	^ port! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:14'!
port: integer
	port := integer! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/11/2011 14:37'!
portOrDefault
	^ self hasPort
		ifTrue: [ self port ]
		ifFalse: [ self class defaultPortForScheme: self schemeOrDefault ]! !

!ZnUrl methodsFor: 'copying' stamp: 'SvenVanCaekenberghe 4/26/2011 12:34'!
postCopy
	super postCopy.
	segments := segments copy.
	query := query copy! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:21'!
printAuthorityOn: stream
	self hasUsername ifTrue: [
		self encode: self username on: stream.
		self hasPassword ifTrue: [
			stream nextPut: $:.
			self encode: self password on: stream ].
		stream nextPut: $@ ].
	self hasHost ifTrue: [ 
		self encode: self host on: stream ].
	self hasPort ifTrue: [ 
		stream nextPut: $:; print: self port ]
! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/14/2013 09:39'!
printOn: stream
	self hasScheme ifTrue: [ 
		stream nextPutAll: self scheme; nextPutAll: '://' ].
	self printAuthorityOn: stream.
	self printPathQueryFragmentOn: stream! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 3/16/2013 19:30'!
printPathOn: stream
	self hasPath ifFalse: [ ^ stream nextPut: $/ ].
 	segments do: [ :each | 
		stream nextPut: $/.
		each == #/
			ifFalse: [ 
				self encode: each on: stream ] ] ! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:34'!
printPathQueryFragmentOn: stream
	self printPathOn: stream.
	self hasQuery
		ifTrue: [ self printQueryOn: stream ].
	self hasFragment
		ifFalse: [ ^ self ].
	stream nextPut: $#.
	self encode: self fragment on: stream! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/8/2012 21:02'!
printQueryOn: stream
	stream nextPut: $?. 
	ZnResourceMetaUtils writeQueryFields: self query on: 	stream! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 16:26'!
query
	^ query! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 20:15'!
query: dictionary
	query := dictionary! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/7/2013 11:14'!
queryAddAll: keyedCollection
	"Note that we use #addAllMulti:"

	keyedCollection isEmpty
		ifFalse: [ 
			query ifNil: [ query := ZnMultiValueDictionary new ].
			query addAllMulti: keyedCollection ].
	^ keyedCollection! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'KenTreis 01/19/2013 12:20'!
queryAt: key
	^ self queryAt: key ifAbsent: [ KeyNotFound signalFor: key printString ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'KenTreis 01/19/2013 12:35'!
queryAt: key add: value
	"Modify the receiver by adding a query variable key=value.
	If key already exists, value is added to it."

	query isNil 
		ifTrue: [ query := ZnMultiValueDictionary new ].
	query at: key asString add: (value ifNotNil: [value asString])! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'KenTreis 01/19/2013 12:16'!
queryAt: key ifAbsent: block
	^ self hasQuery
		ifTrue: [ self query at: key asString ifAbsent: block ]
		ifFalse: [ block value ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'KenTreis 01/19/2013 12:35'!
queryAt: key put: value
	"Modify the receiver by setting a query variable key=value.
	If key is already exists, it is overwritten."

	query isNil 
		ifTrue: [ query := ZnMultiValueDictionary new ].
	query at: key asString put: (value ifNotNil: [value asString])! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/31/2011 12:18'!
queryDo: block
	self hasQuery ifTrue: [ 
		self query keysAndValuesDo: block ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 12/13/2011 14:07'!
queryKeys
	^ self hasQuery
		ifTrue: [ self query keys ]
		ifFalse: [ #() ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/6/2013 19:13'!
queryRemoveAll
	self hasQuery
		ifTrue: [ self query removeAll ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'KenTreis 01/19/2013 12:22'!
queryRemoveKey: key
	self hasQuery
		ifTrue: [ self query removeKey: key asString ifAbsent: [ ] ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 3/16/2013 18:43'!
removeFirstPathSegment
	self hasPath
		ifTrue: [ segments removeFirst ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 3/16/2013 18:42'!
removeLastPathSegment
	self hasPath
		ifTrue: [ segments removeLast ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:09'!
scheme
	^ scheme! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/30/2013 17:18'!
scheme: anObject
	anObject isNil
		ifTrue: [ scheme := nil ]
		ifFalse: [ scheme := anObject asSymbol ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/11/2011 11:05'!
schemeOrDefault
	^ self hasScheme 
		ifTrue: [ self scheme ]
		ifFalse: [ #http ]    ! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 10:56'!
segments
	^ segments! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 13:22'!
segments: collection
	segments := collection! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/11/2011 12:04'!
setDefaults
	self hasScheme 
		ifFalse: [ self scheme: self schemeOrDefault ].
	self hasPort 
		ifFalse: [ self port: self portOrDefault ]    ! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:42'!
username
	^ username! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:19'!
username: string
	username := string! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/9/2012 16:22'!
withPathSegment: segment
	"Return a new URL equal to the receiver with its path extended with segment"
	
	^ self copy
		addPathSegment: segment;
		yourself! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/9/2012 16:23'!
withPathSegments: pathSegments
	"Return a new URL equal to the receiver with its path extended with pathSegments"
	
	^ self copy
		addPathSegments: pathSegments;
		yourself! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/9/2012 16:25'!
withQuery: association
	"Return a new URL equal to the receiver with the association's 
	key=value added as a query parameter.
	Note that #queryAt:add: is used."
	
	^ self copy
		queryAt: association key add: association value;
		yourself 
		! !

Dictionary subclass: #ZnMultiValueDictionary
	instanceVariableNames: 'limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnMultiValueDictionary commentStamp: '<historical>' prior: 0!
I am ZnMultiValueDictionary. 
I am a Dictionary.

I offer an #at:add: method to transparently create Array valued multi entries when needed.
My #keysAndValuesDo: is overwritten to transparently deal with multi entries.
To merge two instance preserving multiple values you can use #addAllMulti:
I normally limit the number of entries to protect me from resource abuse.

Note that most other methods will show the actual value.

Part of Zinc HTTP Components.!

!ZnMultiValueDictionary methodsFor: 'adding' stamp: 'KenTreis 01/19/2013 10:41'!
add: anAssociation
	self checkLimitForKey: anAssociation key.
	^ super add: anAssociation! !

!ZnMultiValueDictionary methodsFor: 'adding' stamp: 'SvenVanCaekenberghe 1/7/2013 10:28'!
addAllMulti: keyedCollection
	"Note that we use #at:add:"

	keyedCollection keysAndValuesDo: [ :key :value | 
		self at: key add: value ]! !

!ZnMultiValueDictionary methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:19'!
at: key add: value
	"Store value under key. If key already exists, store multiple values as Array"
	
	| existingValue newValue |
	existingValue := self at: key ifAbsent: [ nil ].
	newValue := existingValue isNil
		ifTrue: [ value ] 
		ifFalse: [
			existingValue isArray 
				ifTrue: [ existingValue copyWith: value ]
				ifFalse: [ Array with: existingValue with: value ] ].
	^ self at: key put: newValue 

! !

!ZnMultiValueDictionary methodsFor: 'accessing' stamp: 'KenTreis 01/19/2013 10:41'!
at: key put: anObject
	self checkLimitForKey: key.
	^ super at: key put: anObject! !

!ZnMultiValueDictionary methodsFor: 'private' stamp: 'KenTreis 01/19/2013 10:42'!
checkLimitForKey: aKey
	"Signal an exception when the limit, if present, is exceeded."
	
	(self limit notNil and: [ self size >= self limit and: [ (self includesKey: aKey) not ]])
		ifTrue: [ (ZnTooManyDictionaryEntries limit: self limit) signal ].
! !

!ZnMultiValueDictionary methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/8/2012 19:49'!
defaultLimit
	"Return the maximum number dictionary entries to accept.
	Used by ZnMultiValueDictionary and thus for reading headers, 
	url query and application form url encoded entity fields.
	This helps to protect us from malicious content."
	
	^ 256! !

!ZnMultiValueDictionary methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 1/12/2011 11:10'!
keysAndValuesDo: block
	"Execute a two argument (key, value) block for each header.
	Multi-valued headers are handled transparently."

	super keysAndValuesDo: [ :key :value |
		value isArray
			ifTrue: [ 
				value do: [ :each | 
					block value: key value: each ] ]
			ifFalse: [
				block value: key value: value ] ]! !

!ZnMultiValueDictionary methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/8/2012 19:50'!
limit
	^ limit ifNil: [ limit := self  defaultLimit ]! !

!ZnMultiValueDictionary methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/2/2012 19:35'!
limit: numberOfEntries
	limit := numberOfEntries ! !

!ZnMultiValueDictionary methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/2/2012 19:36'!
unlimited
	self limit: nil! !

!Collection methodsFor: '*zinc-resource-meta-core' stamp: 'SvenVanCaekenberghe 3/16/2013 20:24'!
addedToZnUrl: url 
	^ url withPathSegments: self! !

!MIMEType methodsFor: '*zinc-resource-meta-core' stamp: 'SvenVanCaekenberghe 1/4/2011 19:54'!
asZnMimeType
	^ ZnMimeType main: self main sub: self
! !

!String methodsFor: '*zinc-resource-meta-core' stamp: 'SvenVanCaekenberghe 3/16/2013 20:23'!
addedToZnUrl: url
	| segments |
	segments := self findTokens: '/'.
	^ url withPathSegments: segments! !

!String methodsFor: '*zinc-resource-meta-core' stamp: 'SvenVanCaekenberghe 1/4/2011 19:47'!
asZnMimeType
	^ ZnMimeType fromString: self! !

!String methodsFor: '*zinc-resource-meta-core' stamp: 'SvenVanCaekenberghe 1/4/2011 13:03'!
asZnUrl
	^ ZnUrl fromString: self! !

Error subclass: #ZnPortNotANumber
	instanceVariableNames: 'port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnPortNotANumber commentStamp: 'SvenVanCaekenberghe 2/28/2011 15:23' prior: 0!
ZnPortNotANumber is signalled when an URL is parsed containing an illegal port specification.

Part of Zinc HTTP Components.!

!ZnPortNotANumber class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 15:22'!
port: port
	^ self new
		port: port;
		yourself! !

!ZnPortNotANumber methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:22'!
port
	^ port! !

!ZnPortNotANumber methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:22'!
port: anObject
	port := anObject! !

Error subclass: #ZnTooManyDictionaryEntries
	instanceVariableNames: 'limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnTooManyDictionaryEntries commentStamp: 'SvenVanCaekenberghe 1/2/2012 19:22' prior: 0!
ZnTooManyDictionaryEntries is signalled when too many entries are added to a limited ZnMultiValueDictionary.

Part of Zinc HTTP Components. !

!ZnTooManyDictionaryEntries class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/2/2012 19:21'!
limit: limit
	^ self new 
		limit: limit; 
		yourself! !

!ZnTooManyDictionaryEntries methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/2/2012 19:21'!
limit
	^ limit! !

!ZnTooManyDictionaryEntries methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/2/2012 19:21'!
limit: anObject
	limit := anObject! !

Error subclass: #ZnUnknownScheme
	instanceVariableNames: 'scheme'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnUnknownScheme commentStamp: 'DamienPollet 8/4/2011 14:07' prior: 0!
ZnUnknownScheme is signalled when an unknown/unsupported URL scheme is found.

Part of Zinc HTTP Components. !

!ZnUnknownScheme class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/18/2011 13:17'!
scheme: scheme
	^ self new
		scheme: scheme;
		yourself! !

!ZnUnknownScheme methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/18/2011 13:16'!
scheme
	^ scheme! !

!ZnUnknownScheme methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/18/2011 13:16'!
scheme: anObject
	scheme := anObject! !
ZnMimeType initialize!
